"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from jsonpath import JSONPath
from squadcast_sdk import errors, models, utils
from squadcast_sdk._hooks import HookContext
from squadcast_sdk.schedules_export import SchedulesExport
from squadcast_sdk.schedules_overrides import SchedulesOverrides
from squadcast_sdk.types import OptionalNullable, UNSET
from squadcast_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class SchedulesSDK(BaseSDK):
    export: SchedulesExport
    overrides: SchedulesOverrides

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.export = SchedulesExport(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.overrides = SchedulesOverrides(
            self.sdk_configuration, parent_ref=self.parent_ref
        )

    def list(
        self,
        *,
        team_id: str,
        schedule_i_ds: Optional[List[int]] = None,
        participants: Optional[List[str]] = None,
        schedule_name: Optional[str] = None,
        my_on_call: Optional[bool] = None,
        you_and_your_squads: Optional[bool] = None,
        search: Optional[str] = None,
        hide_paused: Optional[bool] = None,
        owner_id: Optional[str] = None,
        escalation_policies: Optional[List[str]] = None,
        without_escalation_policy: Optional[bool] = None,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.SchedulesListSchedulesResponse]:
        r"""List Schedules

        :param team_id:
        :param schedule_i_ds:
        :param participants:
        :param schedule_name:
        :param my_on_call:
        :param you_and_your_squads:
        :param search:
        :param hide_paused:
        :param owner_id:
        :param escalation_policies:
        :param without_escalation_policy:
        :param page_size:
        :param cursor:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesListSchedulesRequest(
            team_id=team_id,
            schedule_i_ds=schedule_i_ds,
            participants=participants,
            schedule_name=schedule_name,
            my_on_call=my_on_call,
            you_and_your_squads=you_and_your_squads,
            search=search,
            hide_paused=hide_paused,
            owner_id=owner_id,
            escalation_policies=escalation_policies,
            without_escalation_policy=without_escalation_policy,
            page_size=page_size,
            cursor=cursor,
        )

        req = self._build_request(
            method="GET",
            path="/v4/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_listSchedules",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.SchedulesListSchedulesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.pageInfo.nextCursor").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size if not request.page_size is None else 0
            if len(results[0]) < limit:
                return None

            return self.list(
                team_id=team_id,
                schedule_i_ds=schedule_i_ds,
                participants=participants,
                schedule_name=schedule_name,
                my_on_call=my_on_call,
                you_and_your_squads=you_and_your_squads,
                search=search,
                hide_paused=hide_paused,
                owner_id=owner_id,
                escalation_policies=escalation_policies,
                without_escalation_policy=without_escalation_policy,
                page_size=page_size,
                cursor=next_cursor,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SchedulesListSchedulesResponse(
                result=unmarshal_json_response(
                    models.SchedulesListSchedulesResponseBody, http_res
                ),
                next=next_func,
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        team_id: str,
        schedule_i_ds: Optional[List[int]] = None,
        participants: Optional[List[str]] = None,
        schedule_name: Optional[str] = None,
        my_on_call: Optional[bool] = None,
        you_and_your_squads: Optional[bool] = None,
        search: Optional[str] = None,
        hide_paused: Optional[bool] = None,
        owner_id: Optional[str] = None,
        escalation_policies: Optional[List[str]] = None,
        without_escalation_policy: Optional[bool] = None,
        page_size: Optional[int] = None,
        cursor: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.SchedulesListSchedulesResponse]:
        r"""List Schedules

        :param team_id:
        :param schedule_i_ds:
        :param participants:
        :param schedule_name:
        :param my_on_call:
        :param you_and_your_squads:
        :param search:
        :param hide_paused:
        :param owner_id:
        :param escalation_policies:
        :param without_escalation_policy:
        :param page_size:
        :param cursor:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesListSchedulesRequest(
            team_id=team_id,
            schedule_i_ds=schedule_i_ds,
            participants=participants,
            schedule_name=schedule_name,
            my_on_call=my_on_call,
            you_and_your_squads=you_and_your_squads,
            search=search,
            hide_paused=hide_paused,
            owner_id=owner_id,
            escalation_policies=escalation_policies,
            without_escalation_policy=without_escalation_policy,
            page_size=page_size,
            cursor=cursor,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_listSchedules",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.SchedulesListSchedulesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.pageInfo.nextCursor").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None
            results = JSONPath("$.data").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.page_size if not request.page_size is None else 0
            if len(results[0]) < limit:
                return None

            return self.list(
                team_id=team_id,
                schedule_i_ds=schedule_i_ds,
                participants=participants,
                schedule_name=schedule_name,
                my_on_call=my_on_call,
                you_and_your_squads=you_and_your_squads,
                search=search,
                hide_paused=hide_paused,
                owner_id=owner_id,
                escalation_policies=escalation_policies,
                without_escalation_policy=without_escalation_policy,
                page_size=page_size,
                cursor=next_cursor,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.SchedulesListSchedulesResponse(
                result=unmarshal_json_response(
                    models.SchedulesListSchedulesResponseBody, http_res
                ),
                next=next_func,
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        description: str,
        team_id: str,
        owner_id: str,
        owner_type: models.V4CreateScheduleRequestOwnerType,
        time_zone: str,
        tags: Union[List[models.V4Tag], List[models.V4TagTypedDict]],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Create Schedule

        :param name:
        :param description:
        :param team_id:
        :param owner_id:
        :param owner_type:
        :param time_zone:
        :param tags:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V4CreateScheduleRequest(
            name=name,
            description=description,
            team_id=team_id,
            owner_id=owner_id,
            owner_type=owner_type,
            time_zone=time_zone,
            tags=utils.get_pydantic_model(tags, List[models.V4Tag]),
        )

        req = self._build_request(
            method="POST",
            path="/v4/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V4CreateScheduleRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_createSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.SchedulesCreateScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        description: str,
        team_id: str,
        owner_id: str,
        owner_type: models.V4CreateScheduleRequestOwnerType,
        time_zone: str,
        tags: Union[List[models.V4Tag], List[models.V4TagTypedDict]],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Create Schedule

        :param name:
        :param description:
        :param team_id:
        :param owner_id:
        :param owner_type:
        :param time_zone:
        :param tags:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V4CreateScheduleRequest(
            name=name,
            description=description,
            team_id=team_id,
            owner_id=owner_id,
            owner_type=owner_type,
            time_zone=time_zone,
            tags=utils.get_pydantic_model(tags, List[models.V4Tag]),
        )

        req = self._build_request_async(
            method="POST",
            path="/v4/schedules",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.V4CreateScheduleRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_createSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.SchedulesCreateScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Delete Schedule

        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesDeleteScheduleRequest(
            schedule_id=schedule_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_deleteSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = utils.stream_to_bytes(http_res)
            return http_res_bytes
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> bytes:
        r"""Delete Schedule

        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesDeleteScheduleRequest(
            schedule_id=schedule_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_deleteSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*/*"):
            http_res_bytes = await utils.stream_to_bytes_async(http_res)
            return http_res_bytes
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_by_id(
        self,
        *,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Get Schedule by ID

        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesGetScheduleByIDRequest(
            schedule_id=schedule_id,
        )

        req = self._build_request(
            method="GET",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_getScheduleById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesGetScheduleByIDResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_by_id_async(
        self,
        *,
        schedule_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Get Schedule by ID

        :param schedule_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesGetScheduleByIDRequest(
            schedule_id=schedule_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_getScheduleById",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesGetScheduleByIDResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        schedule_id: str,
        name: str,
        description: str,
        owner_id: str,
        owner_type: models.V4UpdateScheduleRequestOwnerType,
        tags: Union[List[models.V4Tag], List[models.V4TagTypedDict]],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Update Schedule

        :param schedule_id:
        :param name:
        :param description:
        :param owner_id:
        :param owner_type:
        :param tags:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesUpdateScheduleRequest(
            schedule_id=schedule_id,
            v4_update_schedule_request=models.V4UpdateScheduleRequest(
                name=name,
                description=description,
                owner_id=owner_id,
                owner_type=owner_type,
                tags=utils.get_pydantic_model(tags, List[models.V4Tag]),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_update_schedule_request,
                False,
                False,
                "json",
                models.V4UpdateScheduleRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_updateSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesUpdateScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        schedule_id: str,
        name: str,
        description: str,
        owner_id: str,
        owner_type: models.V4UpdateScheduleRequestOwnerType,
        tags: Union[List[models.V4Tag], List[models.V4TagTypedDict]],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Update Schedule

        :param schedule_id:
        :param name:
        :param description:
        :param owner_id:
        :param owner_type:
        :param tags:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesUpdateScheduleRequest(
            schedule_id=schedule_id,
            v4_update_schedule_request=models.V4UpdateScheduleRequest(
                name=name,
                description=description,
                owner_id=owner_id,
                owner_type=owner_type,
                tags=utils.get_pydantic_model(tags, List[models.V4Tag]),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v4/schedules/{scheduleID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.v4_update_schedule_request,
                False,
                False,
                "json",
                models.V4UpdateScheduleRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_updateSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesUpdateScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def pause_resume(
        self,
        *,
        schedule_id: str,
        action: Optional[models.Action] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4PauseResumeScheduleResponse:
        r"""Pause/Resume Schedule

        :param schedule_id:
        :param action:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesPauseresumeScheduleRequest(
            schedule_id=schedule_id,
            request_body=models.SchedulesPauseresumeScheduleRequestBody(
                action=action,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v4/schedules/{scheduleID}/actions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesPauseresumeScheduleRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_pauseresumeSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesPauseresumeScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def pause_resume_async(
        self,
        *,
        schedule_id: str,
        action: Optional[models.Action] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4PauseResumeScheduleResponse:
        r"""Pause/Resume Schedule

        :param schedule_id:
        :param action:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesPauseresumeScheduleRequest(
            schedule_id=schedule_id,
            request_body=models.SchedulesPauseresumeScheduleRequestBody(
                action=action,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v4/schedules/{scheduleID}/actions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesPauseresumeScheduleRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_pauseresumeSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesPauseresumeScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def change_timezone(
        self,
        *,
        schedule_id: str,
        time_zone: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ChangeTimezoneResponse:
        r"""Change Timezone

        :param schedule_id:
        :param time_zone:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesChangeTimezoneRequest(
            schedule_id=schedule_id,
            request_body=models.SchedulesChangeTimezoneRequestBody(
                time_zone=time_zone,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v4/schedules/{scheduleID}/change-timezone",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesChangeTimezoneRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_changeTimezone",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesChangeTimezoneResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def change_timezone_async(
        self,
        *,
        schedule_id: str,
        time_zone: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ChangeTimezoneResponse:
        r"""Change Timezone

        :param schedule_id:
        :param time_zone:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesChangeTimezoneRequest(
            schedule_id=schedule_id,
            request_body=models.SchedulesChangeTimezoneRequestBody(
                time_zone=time_zone,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v4/schedules/{scheduleID}/change-timezone",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesChangeTimezoneRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_changeTimezone",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SchedulesChangeTimezoneResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def clone(
        self,
        *,
        schedule_id: str,
        request_body: Union[
            models.SchedulesCloneScheduleRequestBody,
            models.SchedulesCloneScheduleRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Clone Schedule

        :param schedule_id:
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesCloneScheduleRequest(
            schedule_id=schedule_id,
            request_body=utils.get_pydantic_model(
                request_body, models.SchedulesCloneScheduleRequestBody
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v4/schedules/{scheduleID}/clone",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesCloneScheduleRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_cloneSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.SchedulesCloneScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def clone_async(
        self,
        *,
        schedule_id: str,
        request_body: Union[
            models.SchedulesCloneScheduleRequestBody,
            models.SchedulesCloneScheduleRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ScheduleResponse:
        r"""Clone Schedule

        :param schedule_id:
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SchedulesCloneScheduleRequest(
            schedule_id=schedule_id,
            request_body=utils.get_pydantic_model(
                request_body, models.SchedulesCloneScheduleRequestBody
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v4/schedules/{scheduleID}/clone",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.SchedulesCloneScheduleRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Schedules_cloneSchedule",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.SchedulesCloneScheduleResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def get_ical_link(
        self,
        *,
        schedule_id: str,
        my_on_call: bool,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ICalLinkResponse:
        r"""Get Schedule ICal Link

        :param schedule_id:
        :param my_on_call:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportGetScheduleIcalLinkRequest(
            schedule_id=schedule_id,
            my_on_call=my_on_call,
        )

        req = self._build_request(
            method="GET",
            path="/v4/schedules/{scheduleID}/ical-link",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Export_getScheduleIcalLink",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ExportGetScheduleIcalLinkResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def get_ical_link_async(
        self,
        *,
        schedule_id: str,
        my_on_call: bool,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ICalLinkResponse:
        r"""Get Schedule ICal Link

        :param schedule_id:
        :param my_on_call:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportGetScheduleIcalLinkRequest(
            schedule_id=schedule_id,
            my_on_call=my_on_call,
        )

        req = self._build_request_async(
            method="GET",
            path="/v4/schedules/{scheduleID}/ical-link",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Export_getScheduleIcalLink",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ExportGetScheduleIcalLinkResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    def create_ical_link(
        self,
        *,
        schedule_id: str,
        my_on_call: bool,
        request_body: Union[
            models.ExportCreateScheduleIcalLinkRequestBody,
            models.ExportCreateScheduleIcalLinkRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ICalLinkResponse:
        r"""Create Schedule ICal Link

        :param schedule_id:
        :param my_on_call:
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportCreateScheduleIcalLinkRequest(
            schedule_id=schedule_id,
            my_on_call=my_on_call,
            request_body=utils.get_pydantic_model(
                request_body, models.ExportCreateScheduleIcalLinkRequestBody
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v4/schedules/{scheduleID}/ical-link",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.ExportCreateScheduleIcalLinkRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Export_createScheduleIcalLink",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ExportCreateScheduleIcalLinkResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)

    async def create_ical_link_async(
        self,
        *,
        schedule_id: str,
        my_on_call: bool,
        request_body: Union[
            models.ExportCreateScheduleIcalLinkRequestBody,
            models.ExportCreateScheduleIcalLinkRequestBodyTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V4ICalLinkResponse:
        r"""Create Schedule ICal Link

        :param schedule_id:
        :param my_on_call:
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ExportCreateScheduleIcalLinkRequest(
            schedule_id=schedule_id,
            my_on_call=my_on_call,
            request_body=utils.get_pydantic_model(
                request_body, models.ExportCreateScheduleIcalLinkRequestBody
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v4/schedules/{scheduleID}/ical-link",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.ExportCreateScheduleIcalLinkRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="Export_createScheduleIcalLink",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "403",
                "404",
                "409",
                "422",
                "4XX",
                "500",
                "502",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.ExportCreateScheduleIcalLinkResponse, http_res
            )
        if utils.match_response(
            http_res,
            ["400", "401", "402", "403", "404", "409", "422"],
            "application/json",
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(
            http_res, ["500", "502", "503", "504"], "application/json"
        ):
            response_data = unmarshal_json_response(errors.CommonV4ErrorData, http_res)
            raise errors.CommonV4Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKDefaultError("API error occurred", http_res, http_res_text)

        raise errors.SDKDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesUpdateStatusPageByIDResponseThemeColorTypedDict(TypedDict):
    primary: str
    secondary: str


class V4StatusPagesUpdateStatusPageByIDResponseThemeColor(BaseModel):
    primary: str

    secondary: str


class V4StatusPagesUpdateStatusPageByIDResponseTypedDict(TypedDict):
    id: int
    name: str
    is_public: bool
    timezone: str
    domain_name: str
    custom_domain_name: str
    contact_email: str
    theme_color: V4StatusPagesUpdateStatusPageByIDResponseThemeColorTypedDict
    allow_components_subscription: bool
    allow_maintenance_subscription: bool
    allow_webhook_subscription: bool
    hide_from_search_engines: bool
    owner_type: str
    owner_id: str
    team_id: str
    organization_id: NotRequired[str]
    description: NotRequired[str]


class V4StatusPagesUpdateStatusPageByIDResponse(BaseModel):
    id: int

    name: str

    is_public: Annotated[bool, pydantic.Field(alias="isPublic")]

    timezone: str

    domain_name: Annotated[str, pydantic.Field(alias="domainName")]

    custom_domain_name: Annotated[str, pydantic.Field(alias="customDomainName")]

    contact_email: Annotated[str, pydantic.Field(alias="contactEmail")]

    theme_color: Annotated[
        V4StatusPagesUpdateStatusPageByIDResponseThemeColor,
        pydantic.Field(alias="themeColor"),
    ]

    allow_components_subscription: Annotated[
        bool, pydantic.Field(alias="allowComponentsSubscription")
    ]

    allow_maintenance_subscription: Annotated[
        bool, pydantic.Field(alias="allowMaintenanceSubscription")
    ]

    allow_webhook_subscription: Annotated[
        bool, pydantic.Field(alias="allowWebhookSubscription")
    ]

    hide_from_search_engines: Annotated[
        bool, pydantic.Field(alias="hideFromSearchEngines")
    ]

    owner_type: Annotated[str, pydantic.Field(alias="ownerType")]

    owner_id: Annotated[str, pydantic.Field(alias="ownerID")]

    team_id: Annotated[str, pydantic.Field(alias="teamID")]

    organization_id: Annotated[
        Optional[str], pydantic.Field(alias="organizationID")
    ] = None

    description: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V4StatusPagesIssuesIssueStateTypedDict(TypedDict):
    id: int
    name: str
    color: str
    slug: str


class V4StatusPagesIssuesIssueState(BaseModel):
    id: int

    name: str

    color: str

    slug: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_components_createcomponentrequest import (
    V4StatusPagesComponentsCreateComponentRequest,
    V4StatusPagesComponentsCreateComponentRequestTypedDict,
)
from .v4_statuspages_components_createcomponentresponse import (
    V4StatusPagesComponentsCreateComponentResponse,
    V4StatusPagesComponentsCreateComponentResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class ComponentsCreateComponentRequestTypedDict(TypedDict):
    statuspage_id: str
    v4_status_pages_components_create_component_request: (
        V4StatusPagesComponentsCreateComponentRequestTypedDict
    )


class ComponentsCreateComponentRequest(BaseModel):
    statuspage_id: Annotated[
        str,
        pydantic.Field(alias="statuspageID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v4_status_pages_components_create_component_request: Annotated[
        V4StatusPagesComponentsCreateComponentRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class ComponentsCreateComponentResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentsCreateComponentResponseTypedDict


class ComponentsCreateComponentResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentsCreateComponentResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_componentgroups_createcomponentgrouprequest import (
    V4StatusPagesComponentGroupsCreateComponentGroupRequest,
    V4StatusPagesComponentGroupsCreateComponentGroupRequestTypedDict,
)
from .v4_statuspages_componentgroups_createcomponentgroupresponse import (
    V4StatusPagesComponentGroupsCreateComponentGroupResponse,
    V4StatusPagesComponentGroupsCreateComponentGroupResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class ComponentGroupsCreateComponentGroupRequestTypedDict(TypedDict):
    statuspage_id: str
    v4_status_pages_component_groups_create_component_group_request: (
        V4StatusPagesComponentGroupsCreateComponentGroupRequestTypedDict
    )


class ComponentGroupsCreateComponentGroupRequest(BaseModel):
    statuspage_id: Annotated[
        str,
        pydantic.Field(alias="statuspageID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v4_status_pages_component_groups_create_component_group_request: Annotated[
        V4StatusPagesComponentGroupsCreateComponentGroupRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class ComponentGroupsCreateComponentGroupResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentGroupsCreateComponentGroupResponseTypedDict


class ComponentGroupsCreateComponentGroupResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4StatusPagesComponentGroupsCreateComponentGroupResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class WebformsRemoveWebformRequestTypedDict(TypedDict):
    webform_id: int
    owner_id: str


class WebformsRemoveWebformRequest(BaseModel):
    webform_id: Annotated[
        int,
        pydantic.Field(alias="webformId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    owner_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=False))
    ]


class WebformsRemoveWebformResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: str


class WebformsRemoveWebformResponse(BaseModel):
    r"""The request has succeeded."""

    data: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_entityowner import (
    V3GlobalEventRulesEntityOwner,
    V3GlobalEventRulesEntityOwnerTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3GlobalEventRulesCreateGlobalEventRuleRequestTypedDict(TypedDict):
    name: str
    description: str
    owner_id: str
    entity_owner: V3GlobalEventRulesEntityOwnerTypedDict


class V3GlobalEventRulesCreateGlobalEventRuleRequest(BaseModel):
    name: str

    description: str

    owner_id: str

    entity_owner: V3GlobalEventRulesEntityOwner



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_teams_teamresponse import V3TeamsTeamResponse, V3TeamsTeamResponseTypedDict
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class TeamsCreateTeamResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3TeamsTeamResponseTypedDict


class TeamsCreateTeamResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: V3TeamsTeamResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V3WorkflowsBulkEnableDisableWorkflowsRequestTypedDict(TypedDict):
    owner_id: str
    enabled: bool
    workflow_ids: List[int]


class V3WorkflowsBulkEnableDisableWorkflowsRequest(BaseModel):
    owner_id: str

    enabled: bool

    workflow_ids: List[int]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_scheduleresponse import V4ScheduleResponse, V4ScheduleResponseTypedDict
from .v4_updateschedulerequest import (
    V4UpdateScheduleRequest,
    V4UpdateScheduleRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class SchedulesUpdateScheduleRequestTypedDict(TypedDict):
    schedule_id: str
    v4_update_schedule_request: V4UpdateScheduleRequestTypedDict


class SchedulesUpdateScheduleRequest(BaseModel):
    schedule_id: Annotated[
        str,
        pydantic.Field(alias="scheduleID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    v4_update_schedule_request: Annotated[
        V4UpdateScheduleRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class SchedulesUpdateScheduleResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4ScheduleResponseTypedDict


class SchedulesUpdateScheduleResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4ScheduleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_rulesetruleresponse import (
    V3GlobalEventRulesRulesetRuleResponse,
    V3GlobalEventRulesRulesetRuleResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesGetRuleByIDRequestTypedDict(TypedDict):
    ger_id: int
    alert_source_version: str
    alert_source_shortname: str
    rule_id: str


class GlobalEventRulesGetRuleByIDRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_version: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alert_source_shortname: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    rule_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GlobalEventRulesGetRuleByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesRulesetRuleResponseTypedDict


class GlobalEventRulesGetRuleByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesRulesetRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class BuildParametersTypedDict(TypedDict):
    circle_job: str


class BuildParameters(BaseModel):
    circle_job: Annotated[str, pydantic.Field(alias="CIRCLE_JOB")]


class PreviousTypedDict(TypedDict):
    build_num: int
    build_time_millis: int
    status: str


class Previous(BaseModel):
    build_num: int

    build_time_millis: int

    status: str


class PreviousSuccessfulBuildTypedDict(TypedDict):
    build_num: int
    build_time_millis: int
    status: str


class PreviousSuccessfulBuild(BaseModel):
    build_num: int

    build_time_millis: int

    status: str


class CircleciResponseTypedDict(TypedDict):
    username: str
    reponame: str
    build_num: int
    build_url: str
    build_parameters: BuildParametersTypedDict
    previous: PreviousTypedDict
    previous_successful_build: PreviousSuccessfulBuildTypedDict
    retry_of: int
    body: str
    subject: str
    status: str
    lifecycle: str
    outcome: str
    committer_date: str
    committer_email: str
    committer_name: str
    author_date: str
    author_email: str
    author_name: str
    branch: str
    vcs_type: str
    vcs_url: str
    start_time: str
    stop_time: str


class CircleciResponse(BaseModel):
    username: str

    reponame: str

    build_num: int

    build_url: str

    build_parameters: BuildParameters

    previous: Previous

    previous_successful_build: PreviousSuccessfulBuild

    retry_of: int

    body: str

    subject: str

    status: str

    lifecycle: str

    outcome: str

    committer_date: str

    committer_email: str

    committer_name: str

    author_date: str

    author_email: str

    author_name: str

    branch: str

    vcs_type: str

    vcs_url: str

    start_time: str

    stop_time: str


class V3IncidentsIncidentActionsCircleCIRebuildResponseTypedDict(TypedDict):
    r"""Represents the response structure for rebuilding a project in CircleCI."""

    action_id: str
    circleci_response: CircleciResponseTypedDict


class V3IncidentsIncidentActionsCircleCIRebuildResponse(BaseModel):
    r"""Represents the response structure for rebuilding a project in CircleCI."""

    action_id: str

    circleci_response: CircleciResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_updatesquadmemberrequest import (
    V4SquadsUpdateSquadMemberRequest,
    V4SquadsUpdateSquadMemberRequestTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    RequestMetadata,
)
from typing_extensions import Annotated, TypedDict


class SquadsUpdateSquadMemberRequestTypedDict(TypedDict):
    squad_id: str
    member_id: str
    replace_with: str
    v4_squads_update_squad_member_request: V4SquadsUpdateSquadMemberRequestTypedDict


class SquadsUpdateSquadMemberRequest(BaseModel):
    squad_id: Annotated[
        str,
        pydantic.Field(alias="squadID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    member_id: Annotated[
        str,
        pydantic.Field(alias="memberID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    replace_with: Annotated[
        str,
        pydantic.Field(alias="replaceWith"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]

    v4_squads_update_squad_member_request: Annotated[
        V4SquadsUpdateSquadMemberRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_users_updateuserrequest import (
    V3UsersUpdateUserRequest,
    V3UsersUpdateUserRequestTypedDict,
)
from .v3_users_userresponse import V3UsersUserResponse, V3UsersUserResponseTypedDict
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class UsersUpdateUserByIDRequestTypedDict(TypedDict):
    user_id: str
    r"""(Required) user id"""
    v3_users_update_user_request: V3UsersUpdateUserRequestTypedDict


class UsersUpdateUserByIDRequest(BaseModel):
    user_id: Annotated[
        str,
        pydantic.Field(alias="userID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""(Required) user id"""

    v3_users_update_user_request: Annotated[
        V3UsersUpdateUserRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class UsersUpdateUserByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3UsersUserResponseTypedDict


class UsersUpdateUserByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3UsersUserResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_issues_componentstatus import (
    V4StatusPagesIssuesComponentStatus,
    V4StatusPagesIssuesComponentStatusTypedDict,
)
from .v4_statuspages_issues_issuecomponentdetail import (
    V4StatusPagesIssuesIssueComponentDetail,
    V4StatusPagesIssuesIssueComponentDetailTypedDict,
)
from .v4_statuspages_issues_issuedetail import (
    V4StatusPagesIssuesIssueDetail,
    V4StatusPagesIssuesIssueDetailTypedDict,
)
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import Annotated, TypedDict


class V4StatusPagesIssuesGetIssueByIDResponseTypedDict(TypedDict):
    id: int
    title: str
    last_updated_at: datetime
    components: List[V4StatusPagesIssuesIssueComponentDetailTypedDict]
    issues: List[V4StatusPagesIssuesIssueDetailTypedDict]
    current_issue_state: V4StatusPagesIssuesComponentStatusTypedDict
    page_status: V4StatusPagesIssuesComponentStatusTypedDict
    under_maintenance: bool
    status_maintenance: V4StatusPagesIssuesComponentStatusTypedDict
    page_id: int
    is_migrated: bool


class V4StatusPagesIssuesGetIssueByIDResponse(BaseModel):
    id: int

    title: str

    last_updated_at: Annotated[datetime, pydantic.Field(alias="lastUpdatedAt")]

    components: List[V4StatusPagesIssuesIssueComponentDetail]

    issues: List[V4StatusPagesIssuesIssueDetail]

    current_issue_state: Annotated[
        V4StatusPagesIssuesComponentStatus, pydantic.Field(alias="currentIssueState")
    ]

    page_status: Annotated[
        V4StatusPagesIssuesComponentStatus, pydantic.Field(alias="pageStatus")
    ]

    under_maintenance: Annotated[bool, pydantic.Field(alias="underMaintenance")]

    status_maintenance: Annotated[
        V4StatusPagesIssuesComponentStatus, pydantic.Field(alias="statusMaintenance")
    ]

    page_id: Annotated[int, pydantic.Field(alias="pageID")]

    is_migrated: Annotated[bool, pydantic.Field(alias="isMigrated")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3ServicesNotificationDelayConfigFixedTimeslotConfigTypedDict(TypedDict):
    start_time: NotRequired[str]
    end_time: NotRequired[str]
    repeat_days: NotRequired[List[int]]


class V3ServicesNotificationDelayConfigFixedTimeslotConfig(BaseModel):
    start_time: Optional[str] = None

    end_time: Optional[str] = None

    repeat_days: Optional[List[int]] = None


class V3ServicesNotificationDelayConfigCustomTimeslotsTypedDict(TypedDict):
    pass


class V3ServicesNotificationDelayConfigCustomTimeslots(BaseModel):
    pass


class V3ServicesNotificationDelayConfigAssignedToTypedDict(TypedDict):
    id: NotRequired[str]
    type: NotRequired[str]


class V3ServicesNotificationDelayConfigAssignedTo(BaseModel):
    id: Optional[str] = None

    type: Optional[str] = None


class V3ServicesNotificationDelayConfigTypedDict(TypedDict):
    is_enabled: bool
    timezone: NotRequired[str]
    fixed_timeslot_config: NotRequired[
        V3ServicesNotificationDelayConfigFixedTimeslotConfigTypedDict
    ]
    custom_timeslots_enabled: NotRequired[bool]
    custom_timeslots: NotRequired[
        V3ServicesNotificationDelayConfigCustomTimeslotsTypedDict
    ]
    assigned_to: NotRequired[V3ServicesNotificationDelayConfigAssignedToTypedDict]


class V3ServicesNotificationDelayConfig(BaseModel):
    is_enabled: bool

    timezone: Optional[str] = None

    fixed_timeslot_config: Optional[
        V3ServicesNotificationDelayConfigFixedTimeslotConfig
    ] = None

    custom_timeslots_enabled: Optional[bool] = None

    custom_timeslots: Optional[V3ServicesNotificationDelayConfigCustomTimeslots] = None

    assigned_to: Optional[V3ServicesNotificationDelayConfigAssignedTo] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_slo_slodetailedresponse import (
    V3SLOSLODetailedResponse,
    V3SLOSLODetailedResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class SLOCreateSLODataTypedDict(TypedDict):
    slo: V3SLOSLODetailedResponseTypedDict


class SLOCreateSLOData(BaseModel):
    slo: V3SLOSLODetailedResponse


class SLOCreateSLOResponseTypedDict(TypedDict):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: SLOCreateSLODataTypedDict


class SLOCreateSLOResponse(BaseModel):
    r"""The request has succeeded and a new resource has been created as a result."""

    data: SLOCreateSLOData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesDeleteGlobalEventRuleByIDRequestTypedDict(TypedDict):
    ger_id: int


class GlobalEventRulesDeleteGlobalEventRuleByIDRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GlobalEventRulesDeleteGlobalEventRuleByIDResponseBodyTypedDict(TypedDict):
    r"""There is no content to send for this request, but the headers may be useful."""


class GlobalEventRulesDeleteGlobalEventRuleByIDResponseBody(BaseModel):
    r"""There is no content to send for this request, but the headers may be useful."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_globaleventruleresponse import (
    V3GlobalEventRulesGlobalEventRuleResponse,
    V3GlobalEventRulesGlobalEventRuleResponseTypedDict,
)
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class GlobalEventRulesGetGlobalEventRuleByIDRequestTypedDict(TypedDict):
    ger_id: int


class GlobalEventRulesGetGlobalEventRuleByIDRequest(BaseModel):
    ger_id: Annotated[
        int, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]


class GlobalEventRulesGetGlobalEventRuleByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesGlobalEventRuleResponseTypedDict


class GlobalEventRulesGetGlobalEventRuleByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V3GlobalEventRulesGlobalEventRuleResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CommonV4PageInfoTypedDict(TypedDict):
    page_size: int
    has_next: bool
    has_previous: bool
    next_cursor: NotRequired[str]
    previous_cursor: NotRequired[str]


class CommonV4PageInfo(BaseModel):
    page_size: Annotated[int, pydantic.Field(alias="pageSize")]

    has_next: Annotated[bool, pydantic.Field(alias="hasNext")]

    has_previous: Annotated[bool, pydantic.Field(alias="hasPrevious")]

    next_cursor: Annotated[Optional[str], pydantic.Field(alias="nextCursor")] = None

    previous_cursor: Annotated[
        Optional[str], pydantic.Field(alias="previousCursor")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_incidents_assignto import V3IncidentsAssignTo, V3IncidentsAssignToTypedDict
from datetime import datetime
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3IncidentsNotificationDelayPolicyTypedDict(TypedDict):
    r"""Policy for delaying notifications."""

    is_notification_delayed: bool
    delayed_until: datetime
    assign_to: V3IncidentsAssignToTypedDict
    r"""Represents the assignment target for delayed notifications."""


class V3IncidentsNotificationDelayPolicy(BaseModel):
    r"""Policy for delaying notifications."""

    is_notification_delayed: bool

    delayed_until: datetime

    assign_to: V3IncidentsAssignTo
    r"""Represents the assignment target for delayed notifications."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v3_globaleventrules_ruleactionupdate import (
    V3GlobalEventRulesRuleActionUpdate,
    V3GlobalEventRulesRuleActionUpdateTypedDict,
)
from squadcast_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class V3GlobalEventRulesUpdateRulesetRequestTypedDict(TypedDict):
    catch_all_action: NotRequired[V3GlobalEventRulesRuleActionUpdateTypedDict]


class V3GlobalEventRulesUpdateRulesetRequest(BaseModel):
    catch_all_action: Optional[V3GlobalEventRulesRuleActionUpdate] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_statuspages_getstatuspagebyidresponse import (
    V4StatusPagesGetStatusPageByIDResponse,
    V4StatusPagesGetStatusPageByIDResponseTypedDict,
)
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class StatusPagesGetStatusPageByIDRequestTypedDict(TypedDict):
    statuspage_id: str


class StatusPagesGetStatusPageByIDRequest(BaseModel):
    statuspage_id: Annotated[
        str,
        pydantic.Field(alias="statuspageID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class StatusPagesGetStatusPageByIDResponseTypedDict(TypedDict):
    r"""The request has succeeded."""

    data: V4StatusPagesGetStatusPageByIDResponseTypedDict


class StatusPagesGetStatusPageByIDResponse(BaseModel):
    r"""The request has succeeded."""

    data: V4StatusPagesGetStatusPageByIDResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


V3ServicesRoutingRulesExpressionBranchOp = Literal[
    "is",
    "is_not",
    "matches",
    "not_contains",
    "gt",
    "lt",
    "gt_eq",
    "lt_eq",
    "call",
    "field_is",
    "field_is_not",
    "field_gt",
    "field_lt",
    "field_gt_eq",
    "field_lt_eq",
]


V3ServicesRoutingRulesExpressionBranchRHSTypedDict = TypeAliasType(
    "V3ServicesRoutingRulesExpressionBranchRHSTypedDict", Union[str, bool, int, float]
)


V3ServicesRoutingRulesExpressionBranchRHS = TypeAliasType(
    "V3ServicesRoutingRulesExpressionBranchRHS", Union[str, bool, int, float]
)


class V3ServicesRoutingRulesExpressionBranchTypedDict(TypedDict):
    lhs: str
    rhs: V3ServicesRoutingRulesExpressionBranchRHSTypedDict
    op: NotRequired[V3ServicesRoutingRulesExpressionBranchOp]


class V3ServicesRoutingRulesExpressionBranch(BaseModel):
    lhs: str

    rhs: V3ServicesRoutingRulesExpressionBranchRHS

    op: Optional[V3ServicesRoutingRulesExpressionBranchOp] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from squadcast_sdk.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class MaintenancesListMaintenancesRequestTypedDict(TypedDict):
    statuspage_id: str
    start_time: str
    end_time: str


class MaintenancesListMaintenancesRequest(BaseModel):
    statuspage_id: Annotated[
        str,
        pydantic.Field(alias="statuspageID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    start_time: Annotated[
        str,
        pydantic.Field(alias="startTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]

    end_time: Annotated[
        str,
        pydantic.Field(alias="endTime"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


V3IncidentsTagsIncidentPriority = Literal[
    "P1",
    "P2",
    "P3",
    "P4",
    "P5",
]
r"""Represents the priority of an incident."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from squadcast_sdk.types import BaseModel
from typing_extensions import Annotated, TypedDict


class ReassignToTypedDict(TypedDict):
    id: str
    type: str


class ReassignTo(BaseModel):
    id: str

    type: str


class V3IncidentsReassignIncidentRequestTypedDict(TypedDict):
    r"""Request body for reassigning an incident."""

    reassign_to: ReassignToTypedDict


class V3IncidentsReassignIncidentRequest(BaseModel):
    r"""Request body for reassigning an incident."""

    reassign_to: Annotated[ReassignTo, pydantic.Field(alias="reassignTo")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsSlackMessageChannelName = Literal["slack_message_channel",]


class V3WorkflowsSlackMessageChannelDataTypedDict(TypedDict):
    channel_id: str
    channel_name: str
    message: str


class V3WorkflowsSlackMessageChannelData(BaseModel):
    channel_id: str

    channel_name: str

    message: str


class V3WorkflowsSlackMessageChannelTypedDict(TypedDict):
    name: V3WorkflowsSlackMessageChannelName
    data: V3WorkflowsSlackMessageChannelDataTypedDict


class V3WorkflowsSlackMessageChannel(BaseModel):
    name: V3WorkflowsSlackMessageChannelName

    data: V3WorkflowsSlackMessageChannelData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
import pydantic
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V4StatusPagesIssuesUpdateIssueRequestComponentTypedDict(TypedDict):
    id: NotRequired[int]
    status_id: NotRequired[int]


class V4StatusPagesIssuesUpdateIssueRequestComponent(BaseModel):
    id: Optional[int] = None

    status_id: Annotated[Optional[int], pydantic.Field(alias="statusID")] = None


class V4StatusPagesIssuesUpdateIssueRequestStateMessageTypedDict(TypedDict):
    text: NotRequired[str]
    timestamp: NotRequired[datetime]


class V4StatusPagesIssuesUpdateIssueRequestStateMessage(BaseModel):
    text: Optional[str] = None

    timestamp: Optional[datetime] = None


class V4StatusPagesIssuesUpdateIssueRequestIssueTypedDict(TypedDict):
    state_id: NotRequired[int]
    state_messages: NotRequired[
        List[V4StatusPagesIssuesUpdateIssueRequestStateMessageTypedDict]
    ]


class V4StatusPagesIssuesUpdateIssueRequestIssue(BaseModel):
    state_id: Annotated[Optional[int], pydantic.Field(alias="stateID")] = None

    state_messages: Annotated[
        Optional[List[V4StatusPagesIssuesUpdateIssueRequestStateMessage]],
        pydantic.Field(alias="stateMessages"),
    ] = None


class V4StatusPagesIssuesUpdateIssueRequestTypedDict(TypedDict):
    title: str
    components: List[V4StatusPagesIssuesUpdateIssueRequestComponentTypedDict]
    issues: List[V4StatusPagesIssuesUpdateIssueRequestIssueTypedDict]
    status_id: NotRequired[int]


class V4StatusPagesIssuesUpdateIssueRequest(BaseModel):
    title: str

    components: List[V4StatusPagesIssuesUpdateIssueRequestComponent]

    issues: List[V4StatusPagesIssuesUpdateIssueRequestIssue]

    status_id: Annotated[Optional[int], pydantic.Field(alias="statusID")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .v4_squads_squadmember import V4SquadsSquadMember, V4SquadsSquadMemberTypedDict
from squadcast_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class V4SquadsCreateSquadRequestTypedDict(TypedDict):
    owner_id: str
    name: str
    members: List[V4SquadsSquadMemberTypedDict]


class V4SquadsCreateSquadRequest(BaseModel):
    owner_id: str

    name: str

    members: List[V4SquadsSquadMember]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesOverlayRenderCustomContentOverlayRequestTypedDict(TypedDict):
    overlay_template_type: str
    template: str
    payload: str


class V3ServicesOverlayRenderCustomContentOverlayRequest(BaseModel):
    overlay_template_type: str

    template: str

    payload: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


V3WorkflowsMsTeamsCreateMeetingLinkName = Literal["msteams_create_meeting_link",]


class V3WorkflowsMsTeamsCreateMeetingLinkTypedDict(TypedDict):
    name: V3WorkflowsMsTeamsCreateMeetingLinkName


class V3WorkflowsMsTeamsCreateMeetingLink(BaseModel):
    name: V3WorkflowsMsTeamsCreateMeetingLinkName



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing_extensions import TypedDict


class V3ServicesServiceTagTypedDict(TypedDict):
    key: str
    value: str


class V3ServicesServiceTag(BaseModel):
    key: str

    value: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from squadcast_sdk.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class V3TeamsAddTeamMemberRequestTypedDict(TypedDict):
    user_id: str
    role_ids: List[str]
    r"""this field is required if you are using RBAC permission model"""
    role: NotRequired[str]
    r"""this field is required if you are using OBAC permission model"""


class V3TeamsAddTeamMemberRequest(BaseModel):
    user_id: str

    role_ids: List[str]
    r"""this field is required if you are using RBAC permission model"""

    role: Optional[str] = None
    r"""this field is required if you are using OBAC permission model"""
